function p = fit_cw_model(mt_angles, offsets, t_mt, tr, lineshape, measurements, t1obs, r1r, fitmask, p0)
%
% p = fit_cw_model(mt_angles, offsets, t_mt, tr, lineshape, measurements, t1obs, [p0, r1r, fitmask])
%
%	A function to fit CW saturation data with the analytical solution for the two-pool model of MT
%		and return two-pool model parameters
%
% output
%	p = structure with fit parameters
%
% input
%	offsets, amplitudes:    saturation amplitudes (in Hz?) and offsets
%	lineshape:              type of solid pool lineshape to be used
%	meassurements:          signal measurements to fit
%	t1obs:                  value of t1obs measured independently
%   p0 (optional):          initial guess for fit parameters (default: [ F R T2f T2r ] = [ 0.1 30 .1 10e-6 ])
%	r1r (optional):         value of r1r (r1b or r1_solid) to be used (default = 1/s)
%	fitmask (optional):		fitting flag for each model parameter [ F R T2f T2r ]
%
% ref: Henkelman et al. Magn Reson Med (1993) 29:759
%

%
%	Ives Levesque, 2011
%	ives@stanford.edu
%
%------------------------------------------------------------------------------


% create initial guess
if ~exist('p0', 'var')
    p0 = 0.1;
    p0(2) = 30;
    p0(3:4) = [.1 12e-6];
end


% default value for r1r is 1 s^-1
if ~exist('r1r','var')
    r1r = 1;
end

% default is to fit all parameters
if ~exist('fitmask','var')
    fitmask = ones(size(p0));
end

% hold on to initial guess as default parameter value for non-estimated parameters
fix_params = p0;
% initialize results arrray
p = p0;

% mask out non-estimated parameters
p0(fitmask==0) = [];


% compute CWPE amplitudes from MT pulse flip angles
omega_cwpe = zeros(size(mt_angles));
for k = 1:length(mt_angles)
	omega_cwpe(k) = cwpe(mt_angles(k), t_mt, tr)
end


% initialize fitting options structure
myopts = optimset('fminsearch');
%optimget(myopts, 'MaxFunEvals')
myopts.MaxFunEvals = 200*5;

display (sprintf('\n fitting'))
[result, fval, fit_flag] = fminsearch(@(p) obj_cw_model(p, measurements, t1obs, lineshape, omega_cwpe, offsets, r1r, fix_params, fitmask), p0, myopts);

% re-fit with increased tolerance if fit_flag is false
while ~fit_flag
	% increase the tolerance
	display (sprintf('\n increasing tolerance\n'))
	myopts = optimset(myopts,'MaxFunEvals', myopts.MaxFunEvals*2);
	%fit_flag
	[result, fval, fit_flag] = fminsearch(@(p) obj_cw_model(p, measurements, t1obs, lineshape, omega_cwpe, offsets, r1r, fix_params, fitmask), p0, myopts);
end


% return results
% insert fitted parameters into results array
p(fitmask==1) = result;

% compute r1f from fit
rd = r1r - 1/t1obs;
kf = p(1)*p(2);
p(5) = 1/t1obs  - kf * rd /(rd + p(2));

% set R1r
p(6) = r1r;


% show fit


plot_fit(offsets, omega_cwpe, lineshape, measurements, p);

