function [m_after, time_after] = applySatPulseMamt(m_before, time_before, pulse_type, flip_angle, my_offset, duration, m0, t1, t2, k_for, k_rev, line_type)
% [m_after, time_after] = applySatPulseMamt(m_before, time_before, pulse_type, flip_angle, duration, m0, t1, t2, k_for, k_rev, line_type)
%
% simulate magnetization of two exchanging components after arbitrary saturation pulse
% uses pseudo-analytical solution from minimal-approximation MT model (MAMT) (see Portnoy & Stanisz, MRM 2007)
% returns resulting magnetization and time
%
%   k_for = 'forward' exchange rate = k12 (a.k.a. kab)
%   k_rev = 'reverse' exchange rate = k21 (a.k.a. kba)
%

%
%   Ives Levesque, 2011
%
%-------------------------------------------------------------------------------------------------------


n_comp = size(m_before,2);

myDEBUG = 0;

if myDEBUG
    fprintf(1,'**************\nIn debug mode!\n**************\n**************\n')
    % perform check of dimensions
    if length(m0) ~= n_comp
        sprintf('Number of M0 values does not match number of components')
        return
    elseif length(t1) ~= n_comp
        sprintf('Number of T1 values does not match number of components')
        return
    elseif length(t2) ~= n_comp
        sprintf('Number of T2 values does not match number of components')
        return
    elseif length(k_for) ~= n_comp - 1
        sprintf('Number of FORWARD k values does not match number of components -1')
        return
    elseif length(k_rev) ~= n_comp - 1
        sprintf('Number of REVERSE k values does not match number of components -1')
        return
    end
end


%----- time resolution stuff --------------------------------------------------------------------------

% MAMT model requires dt = 50e-6 = 50 microseconds;
dt = 50e-6;
n_inter = round(duration/dt);

if myDEBUG
    fprintf(1,'Using %d timepoints\n',n_inter)
end



%---- prepare computation ------------------------------------------------------------------------------

% array to store answer
m_after = m_before;

% compute approximate pulse envelope
my_times = [dt/2:dt:duration];
my_omega1 = feval(pulse_type, flip_angle, duration, my_times);

% off-resonance frequency of saturation pulse - it's the 2*pi*DELTA term from the Henkleman/Sled equations
delta = 2*pi*my_offset;

% compute lineshape response at designated offset frequency
if sum(strcmp(line_type, {'lorentzian' 'gaussian' 'superlorentzian'}))
    line_fcn = [line_type 'Lineshape'];
    line_response(1) = feval(line_fcn,my_offset,t2(2));
else
    error('Unrecognized lineshape_type. Exiting.');
end


%---- compute evolution --------------------------------------------------------------------------------
% compute resulting magnetization in many time steps

m0_vector = reshape([zeros(2,n_comp);m0], 3*n_comp, 1);
m = reshape(m_before,3*n_comp,1);

for interval = 1:n_inter
    w = pi.*my_omega1(interval)^2.*line_response;
    % get bloch equation matrix for CW sat, excluding direct saturation
    acw = getCWSatMatrix(n_comp, t1, t2, k_for, k_rev, w, my_omega1(interval), delta);
    b = diag(reshape([zeros(2,n_comp);1./t1],1,3*n_comp));

    % this is the T1 recovery term
    that_term = -(eye(3*n_comp) - expm(acw*dt))*inv(acw)*b*m0_vector;
    % CW sat of solid pool for interval dt
    m = expm(acw*dt)*m + that_term;
end

% return magnetization vectors in orginal format
m_after = reshape(m,3,n_comp);

% increment time
time_after = time_before + duration;





