function [m_after, time_after] = freePrecession(m_before, time_before, time_lapse, m0, t1, t2, k_for, k_rev)
% [m_after, time_after] = freePrecession(m_before, time_before, time_lapse, m0, t1, t2, k_for, k_rev)
%
% function to simulate the free precession of exchanging magnetization components over defined time period
% uses analytical solution to Bloch-McConnell equations (faster) based on the matrix-exponential solution
% returns resulting magnetization vector and time
%
%   k_for = 'forward' exchange rate = k12 (a.k.a. kab)
%   k_rev = 'reverse' exchange rate = k21 (a.k.a. kba)
%

%
%   Ives Levesque, 2011
%
%-------------------------------------------------------------------------------------------------------


num_components = size(m_before,2);
    
myDEBUG = 0;

if myDEBUG
    fprintf(1,'**************\nIn debug mode!\n**************\n**************\n')

    % perform check of dimensions
    if length(m0) ~= num_components
        sprintf('Number of M0 values does not match number of components')
        return
    elseif length(t1) ~= num_components
        sprintf('Number of T1 values does not match number of components')
        return
    elseif length(t2) ~= num_components
        sprintf('Number of T2 values does not match number of components')
        return
    elseif length(k_for) ~= num_components - 1
        sprintf('Number of FORWARD k values does not match number of components -1')
        return
    elseif length(k_rev) ~= num_components - 1
        sprintf('Number of REVERSE k values does not match number of components -1')
        return
    end

end

%---- prepare computation ------------------------------------------------------------------------------

% array to store answer
m_after = m_before;

% free precession matrix
a = getFreePrecessionMatrix(num_components, t1, t2, k_for, k_rev);

% T1 recovery matrix
b = -diag(reshape([zeros(2,num_components);1./t1],1,3*num_components));

%---- compute evolution --------------------------------------------------------------------------------

% compute resulting magnetization
m0_vector = reshape([zeros(2,num_components);m0], 3*num_components, 1);
m = expm(a*time_lapse)*reshape(m_before,3*num_components,1) + (eye(3*num_components)-expm(a*time_lapse))*m0_vector;
% this is correct as well...  see Cercignani JMR 2008 or Portnoy MRM 2007
%m_too = expm(a*time_lapse)*reshape(m_before,3*num_components,1) + (eye(3*num_components)-expm(a*time_lapse))*inv(a)*b*m0_vector;

% return magnetization vectors in orginal format
m_after = reshape(m,3,num_components);

% increment time
time_after = time_before + time_lapse;

