function [mag_signal, phase_signal, time] = simPulsedSatSpgr(model_system, expt);
% [mag_signal, phase_signal, time] = simPulsedSatSpgr(model_system, expt)
%
% simulation kernel for Pulsed Sat SPGR in two-component system with exchange
% uses shaped pulsed Saturation, "perfect" spoiling, and square excitation pulse, with gradient-echo read-out
% returns total magnetization of liquid component
%
% Allows definition of absorption lineshape_type of each pool ('lorentzian', 'gaussian', 'superlorentzian')
%
% no phase-cycling, but possible
% 

%
%   Ives Levesque, 2011
%
%--------------------------------------------------------------------------------------------------------------------

% parse MT model parameters
M0 = model_system.m0;
t1 = model_system.t1;
t2 = model_system.t2;
k_for = model_system.k_for;
k_rev = model_system.k_rev;
lineshape_type = model_system.lineshape_type;

% define storage arrays
mag_signal = 0;
phase_signal = mag_signal;

% steady-state tolerance - use same as Sled & Pike, JMR 2000
%tolerance = 0.0005;
% use new optimized tolerance level (see pulsed_sat/test_tol_4_pool_pulsed_sat_sim.m)
tolerance = 0.0000075;
%tolerance = 0.0005;

N = length(M0);

%--------------------------------------------------------------------------------------------------------------------
% initial conditions - start fully relaxed
M = [zeros(2,N); M0];

% set time to beginning of saturation pulse train 
time = 0;

%---- apply saturation pulse train --------------------------------------------------------------------------------
display (sprintf('\nflip = %d, offset = %d', expt.sat_flip, expt.sat_offset));                      
display 'saturation train';

num_comp = length(M0);

% determine number of pulses to apply
num_pulses = expt.total_sat_dur/expt.tr;

% determine between-pulse wait time
time_off = expt.tr - expt.sat_dur - expt.excite_dur;

% compute lineshape response at designated offset frequency
if sum(strcmp(lineshape_type, {'lorentzian' 'gaussian' 'superlorentzian'}))
	line_fcn = [lineshape_type 'Lineshape'];
    line_response(1) = feval(line_fcn,expt.sat_offset,t2(2));
else
    error('Unrecognized lineshape_type. Exiting.');
end

% track approach to steady_state 
m_approach = zeros(3,num_comp,num_pulses);
t_approach = zeros(1,num_pulses);


% initiate loop storage variables 
elapsed_inc = time;
result_inc = M;
start = 1;

% apply pulses and relaxation period
for pulse_step = start:num_pulses

    % apply shaped saturation pulse
    [result_inc, elapsed_inc] = applySatPulseMamt(result_inc, elapsed_inc, ...
                                                   expt.sat_pulse_type, expt.sat_flip, expt.sat_offset, expt.sat_dur, ...
                                                   M0, t1, t2, k_for, k_rev, lineshape_type);
    
    % store cumulative results
    m_approach(:,:,pulse_step) = result_inc;
    t_approach(pulse_step) = elapsed_inc;
    
    % apply perfect spoiling for MT sat
    result_inc(1:2,:) = 0;
    
    %check for steady-state
    if exist('old_result') & check_steady_state(old_result, result_inc, [zeros(2,num_comp); M0], tolerance)
        % steady-state reached
        display 'steady-state reached'
        break
    end
    
    % if continuing, save result for next steady-state check
    old_result = result_inc;

    % apply excitation pulse - affects liquid pool only - this could be improved!?!?!
    % in particular, this is probably only accurate for small excitation pulse angles,
    %   and less so for large angles (which might create appreciable MT saturation)
    liquids = 1;
    [result_inc(:,liquids), elapsed_inc] = applySquarePulseQuick(result_inc(:,liquids), elapsed_inc, ...
                                                                expt.excite_flip, expt.excite_dur, 0, M0, t1, t2);

    % free precession period - use analytical (fast) free precession code
    [result_inc, elapsed_inc] = freePrecession(result_inc, elapsed_inc, time_off, M0, t1, t2, k_for, k_rev);
    % apply perfect spoiling for SPGR
    result_inc(1:2,:) = 0;

end

% set output
M = result_inc;
time = elapsed_inc;
                                      
%---- apply final post-sat spoiler gradient - eliminates any residual transverse magnetization created by Sat --------------

M(1:2,:) = 0;

%---- apply excitation pulse --------------------------------------------------------------------------------------
display 'final excite';

%[M, time] = apply_square_pulse_multi(M, time, expt.excite_flip, expt.excite_dur, expt.excite_phase, M0, t1, t2);  
[M, time] = applySquarePulseQuick(M, time, expt.excite_flip, expt.excite_dur, expt.excite_phase, M0, t1, t2);  


%---- read out -----------------------------------------------------------------------------------------------------
display 'sample';

% if non-zero TE is wanted, apply relaxation prior to GRE sampling
%[M, time] = freePrecession(M, time, expt.te, M0, t1, t2, k_for, k_rev);

% sample = assume signal only comes from liquid!!
mag_signal = sqrt(M(1,1)^2 + M(2,1)^2);
phase_signal = atan(M(2,1)/M(1,1));


